"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ajvFactory = exports.AjvCallFactory = void 0;
const ajv_1 = require("ajv");
const typeorm_1 = require("@nestjs/typeorm");
const class_transformer_1 = require("class-transformer");
const constants_1 = require("../../constants");
const utils_1 = require("./utils");
function AjvCallFactory(dataSource, options) {
    const AjvCallInst = new ajv_1.default({ allErrors: true });
    for (const entity of options.entities) {
        const arrayProps = {};
        const relationArrayProps = {};
        const repository = dataSource.getRepository(entity);
        const relations = repository.metadata.relations.map((i) => {
            return i.propertyName;
        });
        const columns = repository.metadata.columns
            .filter((i) => !relations.includes(i.propertyName))
            .map((i) => {
            arrayProps[i.propertyName] = i.isArray;
            return i.propertyName;
        });
        const relationType = repository.metadata.relations.reduce((acum, i) => {
            i.inverseEntityMetadata.target;
            acum[i.propertyName] = i.inverseEntityMetadata.target;
            return acum;
        }, {});
        const fakeObject = columns.reduce((acum, item) => ((acum[item] = ''), acum), {});
        const fieldsArray = Object.keys((0, class_transformer_1.plainToClass)(repository.target, fakeObject));
        const relationsField = repository.metadata.relations.reduce((acum, item) => {
            const relations = item.inverseEntityMetadata.relations.map((i) => i.propertyName);
            const columns = item.inverseEntityMetadata.columns
                .filter((i) => !relations.includes(i.propertyName))
                .map((i) => {
                relationArrayProps[item.propertyName] =
                    relationArrayProps[item.propertyName] || {};
                relationArrayProps[item.propertyName][i.propertyName] = i.isArray;
                return i.propertyName;
            });
            const fakeObject = columns.reduce((acum, item) => ((acum[item] = ''), acum), {});
            acum[item.propertyName] = Object.keys((0, class_transformer_1.plainToClass)(item.inverseEntityMetadata.target, fakeObject));
            return acum;
        }, {});
        const schemaName = entity instanceof Function ? entity.name : entity.options.name;
        AjvCallInst.addSchema((0, utils_1.inputQuerySchema)(entity, fieldsArray, relationsField, `inputQuerySchema-${schemaName}`), `inputQuerySchema-${schemaName}`);
        AjvCallInst.addSchema((0, utils_1.transformQuerySchema)(entity, fieldsArray, relationsField, `transformQuerySchema-${schemaName}`, {
            arrayProps,
            relationArrayProps,
            relationType,
        }), `transformQuerySchema-${schemaName}`);
        AjvCallInst.addSchema((0, utils_1.inputBodyPostSchema)(entity, fieldsArray, relationsField, `inputBodyPostSchema-${schemaName}`, {
            arrayProps,
            relationArrayProps,
            relationType,
        }), `inputBodyPostSchema-${schemaName}`);
        AjvCallInst.addSchema((0, utils_1.inputBodyPatchSchema)(entity, fieldsArray, relationsField, `inputBodyPatchSchema-${schemaName}`, {
            arrayProps,
            relationArrayProps,
            relationType,
        }), `inputBodyPatchSchema-${schemaName}`);
    }
    AjvCallInst.addSchema((0, utils_1.bodyRelationshipSchema)(), 'body-relationship-schema');
    AjvCallInst.addSchema((0, utils_1.bodyRelationshipPatchSchema)(), 'body-relationship-patch-schema');
    return AjvCallInst;
}
exports.AjvCallFactory = AjvCallFactory;
exports.ajvFactory = {
    provide: ajv_1.default,
    useFactory: AjvCallFactory,
    inject: [
        (0, typeorm_1.getDataSourceToken)(),
        {
            token: constants_1.GLOBAL_MODULE_OPTIONS_TOKEN,
            optional: false,
        },
    ],
};
//# sourceMappingURL=ajv.factory.js.map