"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOne = void 0;
const swagger_1 = require("@nestjs/swagger");
const utils_1 = require("../utils");
const class_transformer_1 = require("class-transformer");
const index_1 = require("./index");
function getOne(controller, entity, binding, config) {
    const entityName = entity instanceof Function ? entity.name : entity.options.name;
    const descriptor = Reflect.getOwnPropertyDescriptor(controller.prototype, binding.name);
    if (!descriptor) {
        return;
    }
    const relationsOptions = (0, utils_1.getRelationsOptions)(entity);
    const relationsFields = Object.keys(relationsOptions);
    const columOptions = (0, utils_1.getColumOptions)(entity);
    const fakeObject = Object.keys(columOptions).reduce((acum, item) => {
        acum[item] = '';
        return acum;
    }, {});
    const currentField = Object.keys((0, class_transformer_1.plainToClass)(entity, fakeObject)).map((i) => i);
    (0, swagger_1.ApiParam)({
        name: 'id',
        required: true,
        type: 'integer',
        description: `ID of resource "${entityName}"`,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiQuery)({
        name: 'include',
        required: false,
        enum: relationsFields,
        style: 'simple',
        isArray: true,
        description: `"${entityName}" resource item has been extended with existing relations`,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiQuery)({
        name: 'fields',
        required: false,
        style: 'deepObject',
        schema: {
            type: 'object',
        },
        example: { target: currentField.join(','), [relationsFields[0]]: 'id' },
        description: `Object of field for select field from "${entityName}" resource`,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 404,
        description: `Item of resource "${entityName}" not found`,
        schema: index_1.errorSchema,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 400,
        description: 'Wrong query parameters',
        schema: index_1.errorSchema,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 200,
        description: 'Resource list received successfully',
        schema: Object.assign({}, (0, utils_1.jsonSchemaResponse)(entity, currentField)),
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiOperation)({
        summary: `Get item of resource "${entityName}"`,
        operationId: `${controller.name}_${binding.name}`,
    })(controller.prototype, binding.name, descriptor);
}
exports.getOne = getOne;
//# sourceMappingURL=get-one.js.map