"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchRelationship = void 0;
const utils_1 = require("../utils");
const swagger_1 = require("@nestjs/swagger");
const index_1 = require("./index");
function patchRelationship(controller, entity, binding, config) {
    const entityName = entity instanceof Function ? entity.name : entity.options.name;
    const descriptor = Reflect.getOwnPropertyDescriptor(controller.prototype, binding.name);
    const relationsOptions = (0, utils_1.getRelationsOptions)(entity);
    if (!descriptor) {
        return;
    }
    const dataType = {
        type: 'object',
        properties: {
            type: {
                type: 'string',
            },
            id: {
                type: 'string',
            },
        },
    };
    const schemaType = {
        type: 'object',
        properties: {
            data: {
                oneOf: [
                    dataType,
                    { type: 'null' },
                    {
                        type: 'array',
                        items: dataType,
                    },
                ],
            },
        },
    };
    (0, swagger_1.ApiBody)({
        description: `Item/s of relation for "${entityName}". Request 'data' field depends on relationship type. For "one-to-many" and "many-to-many" relations it should contain an array.`,
        schema: schemaType,
        required: true,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiParam)({
        name: 'id',
        required: true,
        type: 'integer',
        description: `ID of resource "${entityName}"`,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiParam)({
        name: 'relName',
        required: true,
        type: 'string',
        enum: Object.keys(relationsOptions),
        description: `Relation name of resource "${entityName}"`,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 200,
        schema: schemaType,
        description: `Item/s of relation for "${entityName}" has been updated`,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 400,
        description: 'Wrong url parameters',
        schema: index_1.errorSchema,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 422,
        description: 'Incorrect type for relation',
        schema: index_1.errorSchema,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 404,
        description: 'Resource not found ',
        schema: index_1.errorSchema,
    })(controller.prototype, binding.name, descriptor);
    (0, swagger_1.ApiOperation)({
        summary: `Update list of relation for resource "${entityName}"`,
        operationId: `${controller.name}_${binding.name}`,
    })(controller.prototype, binding.name, descriptor);
}
exports.patchRelationship = patchRelationship;
//# sourceMappingURL=patch-relationship.js.map